@@ -48,25 +48,24 @@ attempt.
 """
 from __future__ import absolute_import, division, print_function
 
-__version__ = '0.3.0'
-
 import sys
 import random
 import string
 import subprocess
+import shutil
+import tempfile
 from collections import defaultdict
-from shutil import rmtree
-from tempfile import mkdtemp
 from os.path import join
 
-is_py2 = bool(sys.version_info[0] == 2)
-
-if is_py2:
+if sys.version_info[0] == 2:
     from cStringIO import StringIO
 else:
     from io import StringIO
 
 
+__version__ = '0.3.1'
+
+
 verbose = False
 trials = 5
 
@@ -153,11 +152,119 @@ class Graph(object):
         return True
 
 
+class Hash1(object):
+    """
+    Random hash function generator.
+    For simplicity and speed, this doesn't implement any byte-level hashing
+    scheme.  Instead, a random string is generated and prefixing to key,
+    and then Python's hashing function is used.
+    """
+    def __init__(self, N):
+        self.N = N
+        self.salt = random.randint(0, 1 << 31)
+
+    def DEKhash(self, x, s):
+        for c in s:
+            x = ((x << 5) ^ (x >> 27) ^ ord(c)) % (1 << 31)
+        return x
+
+    def __call__(self, key):
+        return self.DEKhash(self.salt, str(key)) % self.N
+
+    template = """
+def DEKhash(x, s):
+    for c in s:
+        x = ((x << 5) ^ (x >> 27) ^ ord(c)) % (1 << 31)
+    return x
+
+def perfect_hash(key):
+    return (G[DEKhash($S1, key) % $NG] +
+            G[DEKhash($S2, key) % $NG]) % $NG
+"""
+
+class Hash2(object):
+    """
+    Random hash function generator.
+    Simple byte level hashing, each byte is multiplied in sequence to a table
+    containing random numbers modulo N, and then these products are summed up.
+    The table with random numbers is dynamically expanded whenever
+    a key longer than the current table size is encountered.
+    """
+    def __init__(self, N):
+        self.N = N
+        self.salt = []
+
+    def __call__(self, key):
+        while len(self.salt) < len(key): # add more salt as necessary
+            self.salt.append(random.randint(1, self.N - 1))
+
+        return sum(self.salt[i] * ord(c)
+                   for i, c in enumerate(key)) % self.N
+
+    template = """
+S1 = [$S1]
+S2 = [$S2]
+assert len(S1) == len(S2) == $NS
+
+def hash_f(key, T):
+    return sum(T[i % $NS] * ord(c) for i, c in enumerate(key)) % $NG
+
+def perfect_hash(key):
+    return (G[hash_f(key, S1)] + G[hash_f(key, S2)]) % $NG
+"""
+
+class Hash3(object):
+    """
+    Random hash function generator.
+    """
+    def __init__(self, N):
+        self.N = N
+        self.salt = random.randint(32, 127)
+
+    def __call__(self, key):
+        return sum((self.salt ^ ord(c)) * (i + 1)
+                   for i, c in enumerate(key)) % self.N
+
+    template = """
+def hash_f(key, T):
+    return sum((T ^ ord(c)) * (i + 1) for i, c in enumerate(key)) % $NG
+
+def perfect_hash(key):
+    return (G[hash_f(key, $S1)] + G[hash_f(key, $S2)]) % $NG
+"""
+
+class Hash4(object):
+    """
+    Random hash function generator.
+    """
+    chars = string.ascii_letters + string.digits
+
+    def __init__(self, N):
+        self.N = N
+        self.salt = ''
+
+    def __call__(self, key):
+        while len(self.salt) < len(key): # add more salt as necessary
+            self.salt += random.choice(self.chars)
+
+        return sum(ord(self.salt[i]) * ord(c)
+                   for i, c in enumerate(key)) % self.N
+
+    template = """
+def hash_f(key, T):
+    return sum(ord(T[i % $NS]) * ord(c) for i, c in enumerate(key)) % $NG
+
+def perfect_hash(key):
+    return (G[hash_f(key, "$S1")] +
+            G[hash_f(key, "$S2")]) % $NG
+"""
+
+
 class TooManyInterationsError(Exception):
     pass
 
 
-def generate_hash(keys, Hash):
+def generate_hash(keys, Hash=Hash4):
     """
     Return hash functions f1 and f2, and G for a perfect minimal hash.
     Input is an iterable of 'keys', whos indicies are the desired hash values.
@@ -168,6 +275,10 @@ def generate_hash(keys, Hash):
         raise TypeError("list or tuple expected")
     if len(keys) != len(set(keys)):
         raise ValueError("duplicate keys")
+    for key in keys:
+        if not isinstance(key, str):
+            raise TypeError("key a not string: %r" % key)
+
     # N is the number of vertices in the graph G
     N = len(keys) + 1
     if verbose:
@@ -221,117 +332,6 @@ def generate_hash(keys, Hash):
     return f1, f2, G.vertex_values
 
 
-class Hash1(object):
-    """
-    Random hash function generator.
-    For simplicity and speed, this doesn't implement any byte-level hashing
-    scheme.  Instead, a random string is generated and prefixing to str(key),
-    and then Python's hashing function is used.
-    """
-    def __init__(self, N):
-        self.N = N
-        self.salt = random.randint(0, 1 << 31)
-
-    def DEKhash(self, x, s):
-        for c in s:
-            x = ((x << 5) ^ (x >> 27) ^ ord(c)) % (1 << 31)
-        return x
-
-    def __call__(self, key):
-        return self.DEKhash(self.salt, str(key)) % self.N
-
-    template = """
-def DEKhash(x, s):
-    for c in s:
-        x = ((x << 5) ^ (x >> 27) ^ ord(c)) % (1 << 31)
-    return x
-
-def perfect_hash(key):
-    return (G[DEKhash($S1, str(key)) % $NG] +
-            G[DEKhash($S2, str(key)) % $NG]) % $NG
-"""
-
-class Hash2(object):
-    """
-    Random hash function generator.
-    Simple byte level hashing, each byte is multiplied in sequence to a table
-    containing random numbers modulo N, and then these products are summed up.
-    The table with random numbers is dynamically expanded whenever
-    a key longer than the current table size is encountered.
-    """
-    def __init__(self, N):
-        self.N = N
-        self.salt = []
-
-    def __call__(self, key):
-        skey = str(key)
-        while len(self.salt) < len(skey): # add more salt as necessary
-            self.salt.append(random.randint(1, self.N - 1))
-
-        return sum(self.salt[i] * ord(c)
-                   for i, c in enumerate(skey)) % self.N
-
-    template = """
-S1 = [$S1]
-S2 = [$S2]
-assert len(S1) == len(S2) == $NS
-
-def hash_f(key, T):
-    return sum(T[i % $NS] * ord(c) for i, c in enumerate(str(key))) % $NG
-
-def perfect_hash(key):
-    return (G[hash_f(key, S1)] + G[hash_f(key, S2)]) % $NG
-"""
-
-class Hash3(object):
-    """
-    Random hash function generator.
-    """
-    def __init__(self, N):
-        self.N = N
-        self.salt = random.randint(32, 127)
-
-    def __call__(self, key):
-        skey = str(key)
-        return sum((self.salt ^ ord(c)) * (i + 1)
-                   for i, c in enumerate(skey)) % self.N
-
-    template = """
-def hash_f(key, T):
-    return sum((T ^ ord(c)) * (i + 1) for i, c in enumerate(str(key))) % $NG
-
-def perfect_hash(key):
-    return (G[hash_f(key, $S1)] + G[hash_f(key, $S2)]) % $NG
-"""
-
-class Hash4(object):
-    """
-    Random hash function generator.
-    """
-    chars = string.ascii_letters + string.digits
-
-    def __init__(self, N):
-        self.N = N
-        self.salt = ''
-
-    def __call__(self, key):
-        skey = str(key)
-        while len(self.salt) < len(skey): # add more salt as necessary
-            self.salt += random.choice(self.chars)
-
-        return sum(ord(self.salt[i]) * ord(c)
-                   for i, c in enumerate(skey)) % self.N
-
-    template = """
-def hash_f(key, T):
-    return sum(ord(T[i % $NS]) * ord(c) for i, c in enumerate(str(key))) % $NG
-
-def perfect_hash(key):
-    return (G[hash_f(key, "$S1")] +
-            G[hash_f(key, "$S2")]) % $NG
-"""
-
-
 class Format(object):
 
     def __init__(self, width=76, indent=4, delimiter=', '):
@@ -369,7 +369,7 @@ class Format(object):
         return aux.getvalue()
 
 
-def generate_code(keys, template, Hash, options=None):
+def generate_code(keys, template, Hash=Hash4, options=None):
     """
     Takes a list of key value pairs and inserts the generated parameter
     lists into the 'template' strinng.  'Hash' is the random hash function
@@ -478,7 +478,7 @@ for h, k in enumerate(K):
 """
 
 def run_code(code):
-    tmpdir = mkdtemp()
+    tmpdir = tempfile.mkdtemp()
     path = join(tmpdir, 't.py')
     with open(path, 'w') as fo:
         fo.write(code)
@@ -487,7 +487,7 @@ def run_code(code):
     except subprocess.CalledProcessError as e:
         raise AssertionError(e)
     finally:
-        rmtree(tmpdir)
+        shutil.rmtree(tmpdir)
 
 
 def main():
